/*
	File:			LJP_Main.c	

	Description:	Contains the 68K portion of the Palm ARMlet.  It sets up a form so that
					the UI works with the Palm UI, and it calls the ARMlet.

	Author:		

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/


#define winDisplayChangedEvent 0x4101

#include <PalmOS.h>
#include <VFSMgr.h>
#include <StringMgr.h>
#include <PCENativeCall.h>
#include "ljp_pnoload.h"
#include "pnobridge.h"

#include <coretraps.h>

#include <PenInputMgr.h>

#ifdef _SUPPORT_ZODIAC_
	#include "TapWave/TwChars.h"
#endif
	
#include "stdfile.h"
#include "host68k.h"
#include "display68k.h"
#include "pnaTemplateRsc.h"

#include "palm5way.h"

#include "App_Cst.h"
#include "App_FX.h"

#include "LJP_mac.h"


// Define the minimum OS version we support (2.0 for now).
#define kPalmOS10Version	sysMakeROMVersion(1,0,0,sysROMStageRelease,0)

UInt8 g_lowmem;

PnoDescriptor mod_pno;
char mod_started;
char current_mod;
char mod_init;
char mod_launch;

char device_type;
int g_launch_emu;
int path_sys;

extern UInt32 hw_flags;
/**********************************/

void garbage_collect ( void );

void AppStop(int savepref);


void LoadZippedModuleFromCard(char *name);
UInt32 LoadZippedModule(char *name,PnoDescriptor *pno);
UInt32 CallModule(int flags,UInt8 *mod_data,UInt32 size,PnoDescriptor pno);
void UnloadModule(PnoDescriptor pno);
UInt32 UnzipToFtr(char *name,UInt16 Ftr);


/***********************************************************************
 *
 * FUNCTION:    BlankFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the "BlankForm" of this 
 * 				application.
 *
 * PARAMETERS:  eventP - A pointer to the EventType structure.
 *
 * RETURNED:    true if the event was handled and should be passed to
 *				FrmHandleEvent.
 *
 ***********************************************************************/
Boolean BlankFormHandleEvent(EventType * eventP)
{
	Boolean 		handled = false;
	FormType* 		frmP = FrmGetActiveForm();
	UInt32 			flags, pinMgrVersion;
	Err			err = errNone;

	switch (eventP->eType) 
	{
		case frmOpenEvent:
		{	
			g_launch_emu=1;			
			{
			RGBColorType col;
			col.r=0;col.g=0;col.b=0;
			
																
			UIColorSetTableEntry(UIFormFill,&col);
			UIColorSetTableEntry(UIFormFrame,&col);
			
			FrmDrawForm(frmP);
			
			
			/*err = FtrGet( pinCreator, pinFtrAPIVersion, &pinMgrVersion );
			if (!err && pinMgrVersion == pinAPIVersion1_1)
			{
				flags = FtrGet( sysFtrCreator, sysFtrNumInputAreaFlags, &flags );								
				if (flags & grfFtrInputAreaFlagCollapsible)
					PINSetInputAreaState(pinInputAreaClosed);
				if (flags & grfFtrInputAreaFlagCollapsible)
					StatHide();
				if (flags & grfFtrInputAreaFlagCollapsible)
					PINSetInputTriggerState(pinInputTriggerEnabled);											
			}*/	
															
			ljp_WinScreenUnlock();
			
			}
			
			handled = true;
			break;
		}
		case winEnterEvent:	
		case winDisplayChangedEvent:	
//			SysSetOrientation(sysOrientationLandscape);
//			PINSetInputAreaState(pinInputAreaClosed);
		case frmUpdateEvent:
			/*{ljp_WinScreenLock(winLockCopy);
			FrmDrawForm(frmP);
			ljp_WinScreenUnlock();
			}*/			
			handled=true;
			break;
			
		}
	
    
	return handled;
}



/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 * 				handler for the form loaded.
 *
 * PARAMETERS:  eventP - A pointer to the EventType structure.
 *
 * RETURNED:    true if the event was handled and should not be passed
 *				to a higher level handler.
 *
 ***********************************************************************/
Boolean AppHandleEvent(EventType* eventP)
{
	UInt16 formId;
	FormType * frmP= FrmGetActiveForm();
	
	if (eventP->eType == frmLoadEvent)
	{
		/* Load the form resource. */
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);
		/* 
		 * Set the event handler for the form.  The handler of the
		 * currently active form is called by FrmHandleEvent each
		 * time is receives an event. 
		 */
		 
		/*switch (formId)
		{
			case BlankForm:
				FrmSetEventHandler( frmP, BlankFormHandleEvent );
				break;
		}*/
		return true;
	}		
	return false;
}


void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	g_launch_emu=0;

	do 
	{
		/* change timeout if you need periodic nilEvents */
		EvtGetEvent(&event, evtWaitForever);				

		if (! SysHandleEvent(&event))
		{
			if (! AppHandleEvent(&event))
			{
				if (! MenuHandleEvent(0, &event, &error))
				{
					FrmDispatchEvent(&event);
				}	
			}		
		}		
		if (g_launch_emu) 
		{ break;}
	} while (event.eType != appStopEvent);
}

void AppStop(int savepref)
{
	if (!g_launch_emu)
	{
		FrmCloseAllForms();	
	}
			/**/							
}


UInt32 query_heap_free ( void ) {
  UInt16 heapid = 0;
  UInt32 max, bytes;

  MemHeapFreeBytes ( heapid, &bytes, &max );

  //DEBUGS ( "heapfree" );
  //DEBUGU32 ( bytes );

  return ( max );
}


void garbage_collect ( void ) {

  /* on failure, compact memory */
  //MemHeapCompact ( MemPtrHeapID ( true_head() ) );

#ifdef COMPACT_MEM
  UInt16 heap_id;
  MemPtr ptr=MemPtrNew(8);	
  if (!ptr) {DEBUGS("garbage collector error : cannot allocate ptr");return;}
  heap_id=MemPtrHeapID(ptr);
  MemPtrFree(ptr);
  MemHeapCompact ( heap_id );
#endif

  return;
}


/*Boolean HwrEnableDataWrites ( void )
               SYS_TRAP(sysTrapHwrEnableDataWrites);

Boolean HwrDisableDataWrites ( void )
               SYS_TRAP(sysTrapHwrDisableDataWrites); */



void LoadZippedModuleFromCard(char *_name)
{	
  UInt32 processor;
  UInt32 flags = 0;
  ControlType *c;
  UInt8 filetype;
  Err res;
  UInt16 initDelay,old_initDelay,old_initDblDelay;
  UInt16 period,old_period;
  UInt32 old_mask;
  Boolean queueAhead,old_queueAhead;
  MemPtr pnoPtr;
  PnoDescriptor pno;
  char name[256];  
  
  UInt16 		volRefNum;			
  UInt32 		volIterator;
  Err			err = errNone;
  FILE          *f;
  
  // does emu module exist?
  volIterator = vfsIteratorStart;
  while (volIterator != vfsIteratorStop) 
  {
  	err = VFSVolumeEnumerate(&volRefNum, &volIterator);
	if (!err) {
		StrPrintF(name,"%1d:%s%s",volRefNum,VFSROOT,_name);
		f = fopen ( name, "rf" ); // 'f' means: full path specified (not only file name) --Tinnus
    	if ( f ) {
      		break;
      	}
    }
  }
    if (! f ) {
      DEBUGS ( "The application is not present on the card." );
      DEBUGS (name);
      return;
    }
    fclose ( f );
	
  
  
  // clear background
  {
    RectangleType r;
    IndexedColorType c;
    r.topLeft.x = 0;
    r.topLeft.y = 0;
    r.extent.x = 160;
    r.extent.y = 160;
    c = WinSetBackColor ( 0 );
    WinDrawRectangle ( &r, 0 );
    WinSetBackColor ( c );
  }
  // call the PACE Native Object
  FtrGet(sysFtrCreator, sysFtrNumProcessorID, &processor);
  // rearrange memory
  garbage_collect();
  // fire up pno
  if ( sysFtrNumProcessorIsARM ( processor ) )
  {
    pnobridge_t *bridge = (pnobridge_t*) MemPtrNew ( sizeof(pnobridge_t) );
    MemHandle mh;
    UInt8 *tmpPtr;
    UInt16 pitch, width, height;
    Err error;
    UInt32 pnoSize;
    UInt16 version;


	UnzipToFtr(name,FTR_PNO);
	FtrGet(CREATORID, FTR_PNO, (void*)&pnoPtr);		
	
	//check version
	version=((UInt16*)pnoPtr)[0];
	if (version!=VERSION_DB){
		char st[64];
		DEBUGS(".zem file has wrong version number. Please check your install.");
		StrPrintF(st," Version got/wanted : %02X/%02X",version,VERSION_DB);
		DEBUGS(st);
		FtrPtrFree ( CREATORID, FTR_PNO );
		return ;
	} else {
		pnoSize = MemPtrSize(pnoPtr)-4;
		pnoPtr=(MemPtr)((char*)(pnoPtr)+4);
	}

    hw_guess_screen_dimensions ( &width, &height, &pitch );
    bridge -> vfs_max = palm_vfs_count();
    bridge -> vfs_vol = palm_vfs_iter();
    bridge -> vram = (UInt32) hw_query_vram();
    bridge -> eventf = (UInt32) host_event_handler;
    bridge -> display_width = width;
    bridge -> display_height = height;
    bridge -> display_pitch = pitch;
    bridge -> game_rom = 0; //g_autoload;
    bridge -> audio_buf = NULL;
    
    bridge -> flags = (hw_flags&hf_pin_present?BF_HASPIN:0)|
    				(device_type&_ZODIAC_DEVICE_?BF_ZODIAC:0)|
    				(device_type&_HIRES_PLUS_DEVICE_?BF_HIRESPLUS:0);

    res=ljp_PnoLoad ( &pno, pnoPtr,pnoSize );
    FtrPtrFree ( CREATORID, FTR_PNO ); 
    if (res) {
		DEBUGS("Error while loading PNO");
		DEBUGU32(res);
		MemPtrFree(bridge);
		return;    	
    }
	#ifndef _ZIRE31_KEYS_
	old_mask=KeySetMask(~keyBitsAll);		
	#endif
	initDelay=slowestKeyDelayRate;
	period=slowestKeyPeriodRate;
	queueAhead=false;
	KeyRates(false,&old_initDelay,&old_period,&old_initDblDelay,&old_queueAhead);
	KeyRates(true,&initDelay,&period,&initDelay,&queueAhead);
	//read memory to flush D cache
	FLUSH_D_CACHE	
	
#ifndef TESTGUI	
    res = ljp_PnoCall ( &pno, bridge );        

#endif    
    
	//free sys event
	{
		EventType myevent;
		for (;;)
		{
			EvtGetEvent(&myevent,0);
			if (myevent.eType==nilEvent) break;
			//SysHandleEvent(&myevent);
		}
	}
	#ifndef _ZIRE31_KEYS_
	KeySetMask(old_mask/*keyBitsAll*/);
	#endif
	KeyRates(true,&old_initDelay,&old_period,&old_initDblDelay,&old_queueAhead);	
	//unload pno
	res=ljp_PnoUnload ( &pno );
    if (res) {DEBUGS("Error while unloading PNO, probably not enough free ram.");DEBUGS(_name);}    	

    MemPtrFree(bridge);

   // rearrange memory   
    garbage_collect();

  } // is arm and rom picked

}

UInt32 UnzipToFtr(char *name,UInt16 Ftr){
	MemHandle memh;
	MemPtr ptr;
	PnoDescriptor pno;
	UInt32 res;
	pnobridge_t *bridge;	
	UInt16 pitch, width, height;

	memh = DmGetResource('yPNA', PNA_ZLIB);
	if (!memh) {DEBUGS("Error while accessing zlib resource");return 1;}

	ptr = MemHandleLock(memh);

	if (!ptr){DEBUGS("Error while locking zlib resource");return 2;}	

	res=ljp_PnoLoad ( &pno, ptr,MemPtrSize(ptr) );
    FtrPtrFree ( CREATORID, FTR_PNO ); 
    if (res) {DEBUGS("Error while loading zlib pno");return 3;}
	//read memory to flush D cache
	FLUSH_D_CACHE

    bridge = (pnobridge_t*) MemPtrNew ( sizeof(pnobridge_t) );
    hw_guess_screen_dimensions ( &width, &height, &pitch );
    bridge -> vfs_max = palm_vfs_count();
    bridge -> vfs_vol = palm_vfs_iter();
    bridge -> vram = (UInt32) hw_query_vram();
    bridge -> eventf = (UInt32) host_event_handler;
    bridge -> display_width = width;
    bridge -> display_height = height;
    bridge -> display_pitch = pitch;
    bridge -> game_rom = (unsigned long)name;
    bridge -> audio_buf = (unsigned long)Ftr;
    bridge -> flags = BF_UNZIP;

    if (res=ljp_PnoCall ( &pno, bridge )) {DEBUGS("Error while unzipping");DEBUGU32(res);return 4;}
    MemPtrFree(bridge);
	//unload pno

	res=ljp_PnoUnload ( &pno );    		
    if (res) {DEBUGS("Error while unloading PNO(ZIP), probably not enough free ram.");}
   // rearrange memory

	garbage_collect();
	 		
	MemHandleUnlock(memh);
	DmReleaseResource(memh);
}

UInt32 LaunchModuleFromDB(unsigned long creator, unsigned short ID){
	MemHandle memh;
	MemPtr ptr;
	PnoDescriptor pno;
	UInt32 res;
	pnobridge_t *bridge;	
	UInt16 pitch, width, height;

	memh = DmGetResource(creator, ID);
	if (!memh) {DEBUGS("Error while accessing module resource");return 1;}

	ptr = MemHandleLock(memh);

	if (!ptr){DEBUGS("Error while locking module resource");return 2;}	

	res=ljp_PnoLoad ( &pno, ptr,MemPtrSize(ptr) );
    FtrPtrFree ( CREATORID, FTR_PNO ); 
    if (res) {DEBUGS("Error while loading module pno");return 3;}
	//read memory to flush D cache
	FLUSH_D_CACHE

    bridge = (pnobridge_t*) MemPtrNew ( sizeof(pnobridge_t) );
    hw_guess_screen_dimensions ( &width, &height, &pitch );
    bridge -> vfs_max = palm_vfs_count();
    bridge -> vfs_vol = palm_vfs_iter();
    bridge -> vram = (UInt32) hw_query_vram();
    bridge -> eventf = (UInt32) host_event_handler;
    bridge -> display_width = width;
    bridge -> display_height = height;
    bridge -> display_pitch = pitch;
    bridge -> game_rom = 0;
    bridge -> audio_buf = 0;
    bridge -> flags = 0;

    /*if*/ (res=ljp_PnoCall ( &pno, bridge )); /*{DEBUGS("Error while unzipping");DEBUGU32(res);return 4;}*/
    MemPtrFree(bridge);
	//unload pno

	res=ljp_PnoUnload ( &pno );    		
    if (res) {DEBUGS("Error while unloading PNO, probably not enough free ram.");}
   // rearrange memory

	garbage_collect();
	 		
	MemHandleUnlock(memh);
	DmReleaseResource(memh);
}

UInt32 LoadZippedModule(char *_name,PnoDescriptor *_pno){	
  UInt32 processor;
  FormPtr frmP = FrmGetActiveForm();
  UInt8 filetype;
  Err res;
  UInt16 initDelay;
  UInt16 period;
  Boolean queueAhead;
  MemPtr pnoPtr;
  char name[256];  
  
  
  UInt16 		volRefNum;			
  UInt32 		volIterator;
  Err			err = errNone;
  FILE          *f;
  
  // does emu module exist?
  volIterator = vfsIteratorStart;
  while (volIterator != vfsIteratorStop) 
  {
  	err = VFSVolumeEnumerate(&volRefNum, &volIterator);
	if (!err) {
		StrPrintF(name,"%1d:%s%s",volRefNum,VFSROOT,_name);
		f = fopen ( name, "rf" ); // 'f' means: full path specified (not only file name) --Tinnus
    	if ( f ) {
      		break;
      	}
    }
  }
  
  if ( ! f ) {
      DEBUGS ( "File not found" );
      DEBUGS (name);
      return 1;
    }
  fclose(f);
    
  // call the PACE Native Object
  FtrGet(sysFtrCreator, sysFtrNumProcessorID, &processor);

  // rearrange memory
  garbage_collect();

  // fire up pno
  if ( sysFtrNumProcessorIsARM ( processor ) )
  {       	
	UInt32 pnoSize;
	UInt16 version;
  	UnzipToFtr(name,FTR_PNO);

	FtrGet(CREATORID, FTR_PNO, (void*)&pnoPtr);
	//check version
	version=((UInt16*)pnoPtr)[0];
	if (version!=VERSION_DB){
		char st[64];
		DEBUGS(".zem file has wrong version number. Please check your install.");
		StrPrintF(st," Version got/wanted : %02X/%02X",version,VERSION_DB);
		DEBUGS(st);
		FtrPtrFree ( CREATORID, FTR_PNO );
		return -1;
	} else {
		pnoSize = MemPtrSize(pnoPtr)-4;
		pnoPtr=(MemPtr)((char*)(pnoPtr)+4);
	}
	
	
    res=ljp_PnoLoad ( _pno, pnoPtr,pnoSize );
    FtrPtrFree ( CREATORID, FTR_PNO ); 
    if (res) {
		DEBUGS("Error while loading PNO, probably not enough free ram.");
		DEBUGS(_name);
		return 4;
    }
	//read memory to flush D cache
	FLUSH_D_CACHE
	return 0;
  }

  return 4;
}

UInt32 CallModule(int flags,UInt8 *mod_data,UInt32 size,PnoDescriptor _pno)
{
	UInt32 result;
	pnobridge_t *bridge = (pnobridge_t*) MemPtrNew ( sizeof(pnobridge_t) );
	UInt16 pitch, width, height;
	hw_guess_screen_dimensions ( &width, &height, &pitch );
	
    bridge -> vfs_max = palm_vfs_count();
    bridge -> vfs_vol = palm_vfs_iter();
    bridge -> vram = (UInt32) hw_query_vram();
    bridge -> eventf = (UInt32) host_event_handler;
    bridge -> display_width = width;
    bridge -> display_height = height;
    bridge -> display_pitch = pitch;
    bridge -> game_rom = (unsigned long)size;
    bridge -> audio_buf = (unsigned long)mod_data;
    bridge -> flags = flags;
    return ljp_PnoCall ( &_pno, bridge );        
}
	
void UnloadModule(PnoDescriptor _pno)
{
	UInt32 res;
	res=ljp_PnoUnload ( &_pno );    		
    if (res) {DEBUGS("Error while unloading pno");}    	
   // rearrange memory
    garbage_collect(); 	
}


UInt32 PNAMain()
{
	UInt32 res = errNone;
	UInt16 exit_emu=0;
	unsigned short val,valbis;
	UInt32 val32;
	char c;
		
		if ( ! hw_set16bpp() ) {
   			 DEBUGS ( "Couldn't set to 16 bit colour!" );
   			 exit_emu=1;
   		}

   		device_type=0;  		
		{	UInt16 pitch, width, height;	
	    	hw_guess_screen_dimensions ( &width, &height, &pitch );
	    	/*if ((pitch>320)||(width>320)||(height>320)) */
	    	if (/*(pitch>320)||(width>320)||(height>320)*/hw_flags & hf_pin_present) 	    	
	    	{
	    		char text[64];
	    		//StrPrintF(text,"p%dw%dh%d : hi+ detected",pitch,width,height);
	    		//DEBUGS(text);
	    		device_type=_HIRES_PLUS_DEVICE_;
	    	}
	    	
	    	//if ( (width==160) && (height==160) ) ljp_isLowRes=1;
	    }	    	    
	    if (is_zodiac()) device_type|=_ZODIAC_DEVICE_;
   		if (is_clie()) device_type|=_CLIE_DEVICE_;
   		

		if (!exit_emu)
		{						
			g_lowmem=0;
			
			for (;;)
			{			
				/*if (FrmGetActiveFormID()!=BlankForm) {
					FrmGotoForm( BlankForm );
				}
				else {
					FrmUpdateForm( BlankForm ,0);
				}
			
				AppEventLoop();
				AppStop(1);			*/
																												
//				if (g_launch_emu) 
				{									
					char *ptr;
					UInt32 val;													
  					  					  					  					
					RectangleType rclip;
					UInt16 err;
					WinHandle runningWnd,oldWnd;
					rclip.topLeft.x=0;
					rclip.topLeft.y=0;
					rclip.extent.x=160;
					rclip.extent.y=160;
					oldWnd=WinGetActiveWindow();
					runningWnd=WinCreateWindow( &rclip,noFrame,false,true, &err );
					if (err) runningWnd=NULL;									
					
					if (runningWnd)
					{
					  {
							//launch module...
							LaunchModuleFromDB('yPNA',PNA_ARM);
						}
					

					if (runningWnd) WinDeleteWindow(runningWnd,false);
					else
					{menu_inform("cannot free window");}
					WinSetActiveWindow(oldWnd);
					}	
					
/*					
				}
				else 
				{ */
					//FrmCloseAllForms();	
					break; //tmp bug fixes due to snes bug when snes launched 2 times
				}
				
			}								

        }


}

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion <= kPalmOS10Version)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    NativeFuncType* entry;
    UInt32 res;
	
	if (cmd == sysAppLaunchCmdNormalLaunch) 
	{	
		res=0;
		if (!RomVersionCompatible(kOurMinVersion, launchFlags)) 
			res=PNAMain(0);
	}
		
	return res;
}


